package edu.tum.motras.legacy.pmtsrc.util.formula;

import java.util.ArrayList;
import java.util.Deque;
import java.util.LinkedList;
import java.util.List;

public class Composition extends TreeBoolean {

	public static final boolean CONJUNCTION = true;
	public static final boolean DISJUNCTION = false;

	boolean type;
	final TreeBoolean[] children;

	private Composition(boolean t, TreeBoolean[] c) {
		children = c;
		type = t;
	}

	public TreeBoolean[] getChildren() {
		return children;
	}

	@Override
	public String toString() {
		String result = (type ? "&" : "|") + " [\n";

		for (TreeBoolean child : children) {
			result += "  " + child.toString().replace("\n", "\n  ") + "\n";
		}

		return result + "]";
	}

	@Override
	public <R> R accept(Visitor<R> visitor) {
		return visitor.visit(this);
	}

	static TreeBoolean createComposition(boolean type, TreeBoolean... children) {
		List<TreeBoolean> finished = new ArrayList<>();
		Deque<TreeBoolean[]> merger = new LinkedList<>();

		merger.add(children);

		while (!merger.isEmpty()) {
			TreeBoolean[] current = merger.removeFirst();

			for (TreeBoolean child : current) {
				if (child instanceof Literal) {
					Literal l = (Literal) child;

					if (l.variable != -1) {
						finished.add(l);
					} else if (l.isTrue() && type == DISJUNCTION) {
						return l;
					} else if (l.isFalse() && type == CONJUNCTION) {
						return l;
					}
				} else {
					Composition composition = (Composition) child;

					if (composition.type == type) {
						merger.add(composition.children);
					} else {
						finished.add(composition);
					}
				}
			}
		}

		if (finished.size() == 0 && type == CONJUNCTION) {
			return new Literal(true);
		}

		if (finished.size() == 0 && type == DISJUNCTION) {
			return new Literal(false);
		}

		if (finished.size() == 1) {
			return finished.iterator().next();
		}

		return new Composition(type, Util.toTreeArray(finished));
	}

	@Override
	public void accept(VoidVisitor visitor) {
		visitor.visit(this);
	}

	@Override
	public TreeBoolean clone() {
		TreeBoolean[] cloneChildren = new TreeBoolean[children.length];
		int i = 0;

		for (TreeBoolean child : children) {
			cloneChildren[i] = child.clone();
			i++;
		}

		return new Composition(type, cloneChildren);
	}
}