package edu.tum.motras.legacy.pmtsrc.util.formula;

import java.util.Collection;

public abstract class TreeBoolean implements Cloneable {

	abstract public <R> R accept(Visitor<R> visitor);

	abstract public void accept(VoidVisitor visitor);

	public interface Visitor<R> {
		public R visit(Literal l);

		public R visit(Composition c);
	}

	public interface VoidVisitor {
		public void visit(Literal l);

		public void visit(Composition c);
	}

	public static TreeBoolean and(Collection<TreeBoolean> c) {
		return Composition.createComposition(Composition.CONJUNCTION, Util.toTreeArray(c));
	}

	public static TreeBoolean and(TreeBoolean... c) {
		return Composition.createComposition(Composition.CONJUNCTION, c);
	}

	public static TreeBoolean or(Collection<TreeBoolean> c) {
		return Composition.createComposition(Composition.DISJUNCTION, Util.toTreeArray(c));
	}

	public static TreeBoolean or(TreeBoolean... c) {
		return Composition.createComposition(Composition.DISJUNCTION, c);
	}

	public TreeBoolean and(TreeBoolean f) {
		return Composition.createComposition(Composition.CONJUNCTION, this, f);
	}

	public TreeBoolean or(TreeBoolean f) {
		return Composition.createComposition(Composition.DISJUNCTION, this, f);
	}

	public TreeBoolean not() {
		this.accept(notVisitor);
		return this;
	}

	public TreeBoolean iff(TreeBoolean f) {
		return (this.and(f)).or((this.clone()).not().and((f.clone()).not()));
	}

	public TreeBoolean implies(TreeBoolean f) {
		return (this.not()).or(f);
	}

	private static NotVisitor notVisitor = new NotVisitor();

	protected static class NotVisitor implements VoidVisitor {

		@Override
		public void visit(Literal l) {
			l.negated = !l.negated;
		}

		@Override
		public void visit(Composition c) {
			c.type = !c.type;

			for (TreeBoolean child : c.children) {
				child.accept(this);
			}
		}
	}

	@Override
	abstract public TreeBoolean clone();
}
