package edu.tum.xmts;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.jdt.annotation.NonNull;

public abstract class GenericRecord<T> {

	/* XMTS Properties */
	public static final String PROP_ID = "id";
	public static final String PROP_LABEL = "label";

	protected @NonNull String id;
	protected String label;

	/* Events */
	protected transient final PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

	/* Parent of the Record */
	protected final @NonNull T context;

	protected GenericRecord(@NonNull T context, @NonNull String id) {
		this.context = context;
		this.id = id;
	}

	public @NonNull T getContext() {
		return context;
	}

	public @NonNull String getId() {
		return id;
	}

	public void setId(String id) throws IllegalArgumentException {
		if (!id.matches("^[a-zA-Z][a-zA-Z_0-9]*$")) {
			throw new IllegalArgumentException("Malformed ID: " + id);
		}

		if (this.id.equals(id)) {
			return;
		}

		if (this instanceof Parameter || this instanceof Action || this instanceof State) {
			TransitionSystem otherContext = (TransitionSystem) this.getContext();

			if (!otherContext.isUniqueId(id)) {
				throw new IllegalArgumentException("ID is not unique in the context of this system");
			}
		}

		String oldId = this.id;
		this.id = id;
		propertyChangeSupport.firePropertyChange(PROP_ID, oldId, id);
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		if (label != null && label.contains("\"")) {
			throw new IllegalArgumentException("Found forbidden \" symbol in label");
		}

		String oldLabel = this.label;
		this.label = label;
		propertyChangeSupport.firePropertyChange(PROP_LABEL, oldLabel, label);
	}

	@Override
	public String toString() {
		return id;
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.removePropertyChangeListener(listener);
	}

	public boolean isSupportedProperty(@NonNull String propName) {
		return propName.equals(PROP_ID) || propName.equals(PROP_LABEL);
	}
}
