package edu.tum.xmts;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

@NonNullByDefault
public class Transition extends GenericRecord<State> {

	public static class Duration {
		public enum Type {
			CONTROLLABLE, UNCONTROLLABLE
		}

		public final Type type;
		public final int lower;
		public final int upper;

		Duration(Type type, int lower, int upper) {
			this.type = type;
			this.lower = lower;
			this.upper = upper;
		}

		@Override
		public String toString() {
			if (type == Type.UNCONTROLLABLE) {
				return "[" + lower + "," + upper + "]";
			}

			return "(" + lower + "," + upper + ")";
		}
	}

	/* XMTS Properties */

	public static final String PROP_DURATION = "duration";

	private final Action action;
	private final State state;
	private @Nullable Duration duration;

	Transition(State context, Action action, State state) {
		super(context, context.id + "." + action.id + "." + state.id);

		this.action = action;
		this.state = state;
		this.duration = null;
	}

	public void clearDuration() {
		Duration oldDuration = duration;
		duration = null;
		propertyChangeSupport.firePropertyChange(PROP_DURATION, oldDuration, duration);
	}

	public Action getAction() {
		return action;
	}

	public @Nullable Duration getDuration() {
		return duration;
	}

	public State getState() {
		return state;
	}

	public void setDuration(Duration.Type type, int lower, int upper) {
		Duration oldDuration = duration;
		duration = new Duration(type, lower, upper);
		propertyChangeSupport.firePropertyChange(PROP_DURATION, oldDuration, duration);
	}

	@Override
	public void setId(@Nullable String id) {
		throw new UnsupportedOperationException("The Id of transitions is immutable");
	}

	@Override
	public boolean isSupportedProperty(String propName) {
		if (super.isSupportedProperty(propName)) {
			return true;
		}

		TransitionSystem system = getContext().getContext();

		if (system.getType() != TransitionSystem.Type.MTSD && system.getType() != TransitionSystem.Type.MTSD_DPS) {
			return false;
		}

		return propName.equals(PROP_DURATION);
	}
}
