package edu.tum.xmts.formula;

import java.util.Set;

import edu.tum.xmts.Transition;
import edu.tum.xmts.formula.Formula.Atom;
import edu.tum.xmts.formula.Formula.Conjunction;
import edu.tum.xmts.formula.Formula.Disjunction;
import edu.tum.xmts.formula.Formula.Equivalence;
import edu.tum.xmts.formula.Formula.Implication;
import edu.tum.xmts.formula.Formula.Negation;

public class EvaluateVisitor implements BooleanVisitor {

	private final Set<Transition> assignment;

	public EvaluateVisitor(Set<Transition> set) {
		this.assignment = set;
	}

	@Override
	public boolean visit(Atom<?> atom) {
		Object object = atom.getPayload();

		if (atom == Formula.TRUE) {
			return true;
		}

		if (atom == Formula.FALSE) {
			return false;
		}

		return new Boolean(assignment.contains(object));
	}

	@Override
	public boolean visit(Conjunction conjunction) {
		for (Formula child : conjunction.children) {
			if (!child.accept(this)) {
				return false;
			}
		}

		return true;
	}

	@Override
	public boolean visit(Disjunction disjunction) {
		for (Formula child : disjunction.children) {
			if (child.accept(this)) {
				return true;
			}
		}

		return false;
	}

	@Override
	public boolean visit(Equivalence equivalence) {
		return equivalence.children.get(0).accept(this) ^ (!equivalence.children.get(1).accept(this));
	}

	@Override
	public boolean visit(Implication implication) {
		if (!implication.children.get(0).accept(this)) {
			return true;
		}

		return implication.children.get(1).accept(this);
	}

	@Override
	public boolean visit(Negation negation) {
		return !negation.children.get(0).accept(this);
	}
}