package edu.tum.motras.algorithms.junit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.Deque;
import java.util.LinkedList;

import org.junit.Before;
import org.junit.Test;

import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.Algorithm;
import edu.tum.motras.algorithms.CompositionDMTS;
import edu.tum.motras.algorithms.CompositionMTS;
import edu.tum.motras.algorithms.MaxImplementation;
import edu.tum.motras.algorithms.ModalRefinementQSat;
import edu.tum.xmts.State;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;
import edu.tum.xmts.utils.XMTSGenerator;
import edu.tum.xmts.utils.XMTSGenerator.XMTSGeneratorConfiguration;

public class CompositionDMTSTest {

	CompositionDMTS compositionDMTS;

	@Before
	public void setUp() throws Exception {
		compositionDMTS = new CompositionDMTS();
		compositionDMTS.setConfiguration("a");
	}

	public static LinkedList<TransitionSystem> generate(Type type) {
		LinkedList<TransitionSystem> systems = new LinkedList<>();
		XMTSGenerator generator = new XMTSGenerator();
		XMTSGeneratorConfiguration configuration = new XMTSGeneratorConfiguration();

		configuration.type = type;
		configuration.parameters = 0;
		configuration.actions = 3;

		configuration.clusterInterfaceNodes = 1;
		configuration.clusterNodeDegree = 0;
		configuration.clusterSize = 1;

		configuration.systemNodeDegree = 3;
		configuration.systemSize = 7;

		configuration.deterministic = false;

		if (type == Type.PMTS) {
			configuration.parameters = 2;
		}

		for (int i = 0; i < 6; i++) {
			systems.add(generator.generate(configuration, 13l + i));
		}

		return systems;
	}

	@Test
	public void testApplyRandom() throws Exception {
		Algorithm<Boolean> modalRefinement = new ModalRefinementQSat();
		CompositionDMTS composition1 = new CompositionDMTS("a, a0, a0");
		CompositionMTS composition2 = new CompositionMTS("a, a0, a0");
		Deque<TransitionSystem> transitionSystems = generate(Type.DMTS);
		transitionSystems.addAll(generate(Type.MTS));
		MaxImplementation maxImplementation1 = new MaxImplementation();
		MaxImplementation maxImplementation2 = new MaxImplementation();

		assertEquals(ImmutableSet.of("a0", "a"), composition1.getSynchronizingAlphabet());

		while (transitionSystems.size() > 1) {
			TransitionSystem transitionSystem1 = transitionSystems.pop();
			TransitionSystem transitionSystem2 = transitionSystems.pop();

			State abstract1 = transitionSystem1.states.getElements().get(0);
			State abstract2 = transitionSystem2.states.getElements().get(0);

			State implementation1 = maxImplementation1.apply(abstract1);
			State implementation2 = maxImplementation2.apply(abstract2);

			State compAbstract = composition1.apply(abstract1, abstract2);
			State compImplementation = composition2.apply(implementation1, implementation2);

			assertTrue(modalRefinement.apply(implementation1, abstract1));
			assertTrue(modalRefinement.apply(implementation2, abstract2));
			assertTrue(modalRefinement.apply(compImplementation, compAbstract));
		}
	}

	@Test
	public void testGetArity() {
		AlgorithmTestUtil.testGetArity(compositionDMTS, 2);
	}

	@Test
	public void testGetSupportedTypes() {
		AlgorithmTestUtil.testGetSupportedTypes(compositionDMTS, ImmutableSet.of(Type.MTS, Type.DMTS));
	}
}
