package edu.tum.motras.algorithms.junit;

import static org.junit.Assert.assertTrue;

import org.junit.Before;
import org.junit.Test;

import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.Consistency;
import edu.tum.xmts.State;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;
import edu.tum.xmts.utils.XMTSReader;

public class ConsistencyTest {

	Consistency consistency;

	@Before
	public void setUp() throws Exception {
		consistency = new Consistency();
	}

	@Test
	public void testApply() throws Exception {
		String xmts1 = "mts test1[action a action b action c " + "state s1 [obligation all  a->s2]"
				+ "state s2 [obligation true b->s1 c->s1]]";

		String xmts2 = "mts test2[action a action b action c " + "state s1 [obligation true a->s2 a->s3]"
				+ "state s2 [obligation all  b->s1]" + "state s3 [obligation all  c->s1]]";

		XMTSReader reader = new XMTSReader();
		TransitionSystem system1 = reader.read(xmts1);
		TransitionSystem system2 = reader.read(xmts2);

		for (State state : system2.states.getElements()) {
			assertTrue(consistency.apply(state, state));
		}

		assertTrue(consistency.apply(system1.states.lookup("s1"), system2.states.lookup("s1")));
		assertTrue(consistency.apply(system2.states.lookup("s1"), system1.states.lookup("s1")));
	}

	@Test
	public void testGetArity() {
		AlgorithmTestUtil.testGetArity(consistency, 2);
	}

	@Test
	public void testGetSupportedTypes() {
		AlgorithmTestUtil.testGetSupportedTypes(consistency, ImmutableSet.of(Type.MTS, Type.DMTS));
	}

}
