package edu.tum.motras.algorithms.junit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.util.Deque;

import org.junit.Before;
import org.junit.Test;

import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.ModalRefinementFixedPoint;
import edu.tum.motras.algorithms.ModalRefinementQSat;
import edu.tum.xmts.State;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;
import edu.tum.xmts.utils.XMTSReader;

public class ModalRefinementFixedPointTest {

	ModalRefinementFixedPoint modalRefinementFixedPoint;

	@Before
	public void setUp() throws Exception {
		modalRefinementFixedPoint = new ModalRefinementFixedPoint();
	}

	@Test
	public void testApply() throws Exception {
		XMTSReader reader = new XMTSReader();

		State state1 = reader.read(new File("../xmts-specification/examples/coffee and tea/CoffeeImplementation.xmts"))
				.getInitialState();
		State state2 = reader.read(new File("../xmts-specification/examples/coffee and tea/CoffeeModel.xmts"))
				.getInitialState();

		assertTrue(modalRefinementFixedPoint.apply(state1, state1));
		assertTrue(modalRefinementFixedPoint.apply(state1, state2));
		assertFalse(modalRefinementFixedPoint.apply(state2, state1));
		assertTrue(modalRefinementFixedPoint.apply(state2, state2));
	}

	@Test
	public void testApplyRandom() throws Exception {
		ModalRefinementQSat modalRefinementQSat = new ModalRefinementQSat();
		Deque<TransitionSystem> transitionSystems = AlgorithmTestUtil.generate(Type.MTS);

		while (transitionSystems.size() > 1) {
			TransitionSystem system1 = transitionSystems.removeFirst();
			TransitionSystem system2 = transitionSystems.removeFirst();

			for (State state1 : system1.states.getElements()) {
				for (State state2 : system2.states.getElements()) {
					assertEquals(modalRefinementQSat.apply(state1, state2),
							modalRefinementFixedPoint.apply(state1, state2));
				}
			}
		}
	}

	@Test
	public void testGetArity() {
		AlgorithmTestUtil.testGetArity(modalRefinementFixedPoint, 2);
	}

	@Test
	public void testGetSupportedTypes() {
		AlgorithmTestUtil.testGetSupportedTypes(modalRefinementFixedPoint, ImmutableSet.of(Type.MTS));
	}

}
