package edu.tum.motras.algorithms.junit;

import static org.junit.Assert.assertTrue;

import org.junit.Before;
import org.junit.Test;

import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.Algorithm;
import edu.tum.motras.algorithms.ModalRefinementQSat;
import edu.tum.motras.algorithms.RandomRefinement;
import edu.tum.xmts.State;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;

public class RandomRefinementTest {

	private RandomRefinement randomRefinement;
	private Algorithm<Boolean> modalRefinement;

	@Before
	public void setUp() throws Exception {
		randomRefinement = new RandomRefinement();
		modalRefinement = new ModalRefinementQSat();
	}

	@Test
	public void testApplyRandom() throws Exception {
		for (TransitionSystem transitionSystem : AlgorithmTestUtil.generate(Type.MTS)) {
			State s1 = transitionSystem.states.getElements().get(0);
			State s1refine = randomRefinement.apply(s1);
			assertTrue(modalRefinement.apply(s1refine, s1));
		}

		for (TransitionSystem transitionSystem : AlgorithmTestUtil.generate(Type.DMTS)) {
			State s1 = transitionSystem.states.getElements().get(0);
			State s1refine = randomRefinement.apply(s1);
			assertTrue(modalRefinement.apply(s1refine, s1));
		}

		for (TransitionSystem transitionSystem : AlgorithmTestUtil.generate(Type.BMTS)) {
			State s1 = transitionSystem.states.getElements().get(0);
			State s1refine = randomRefinement.apply(s1);
			assertTrue(modalRefinement.apply(s1refine, s1));
		}

		for (TransitionSystem transitionSystem : AlgorithmTestUtil.generate(Type.PMTS)) {
			State s1 = transitionSystem.states.getElements().get(0);
			State s1refine = randomRefinement.apply(s1);
			assertTrue(modalRefinement.apply(s1refine, s1));
		}
	}

	@Test
	public void testGetArity() {
		AlgorithmTestUtil.testGetArity(randomRefinement, 1);
	}

	@Test
	public void testGetSupportedTypes() {
		AlgorithmTestUtil.testGetSupportedTypes(randomRefinement,
				ImmutableSet.of(Type.MTS, Type.DMTS, Type.BMTS, Type.PMTS));
	}

}
