package edu.tum.xmts.junit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import org.junit.Before;
import org.junit.Test;

import edu.tum.xmts.State;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;

public class TransitionSystemTest {

	TransitionSystem transitionSystem;

	@Before
	public void setUp() throws Exception {
		transitionSystem = new TransitionSystem("foo");
	}

	@Test
	public void testPool() {
		State state = transitionSystem.states.create("fooState");
		state.setId("barState");
		assertEquals(state, transitionSystem.states.lookup("barState"));
	}

	@Test
	public void testInitialState() {
		State state = transitionSystem.states.create();
		transitionSystem.setInitialState(state);
		assertEquals(state, transitionSystem.getInitialState());
		transitionSystem.clearInitialState();
		assertEquals(null, transitionSystem.getInitialState());
	}

	@Test
	public void testType() {
		transitionSystem.setType(Type.DMTS);
		assertEquals(Type.DMTS, transitionSystem.getType());
		transitionSystem.setType(Type.MTS);
		assertEquals(Type.MTS, transitionSystem.getType());
	}

	@Test
	public void testId() {
		transitionSystem.setId("bar");
		assertEquals("bar", transitionSystem.getId());

		try {
			transitionSystem.setId("234!§12");
		} catch (IllegalArgumentException ex) {
			return;
		}

		fail("Missing exception");
	}

	@Test
	public void testLabel() {
		transitionSystem.setLabel("long description.... foo barz");
		assertEquals("long description.... foo barz", transitionSystem.getLabel());
		transitionSystem.setLabel(null);
		assertEquals(null, transitionSystem.getLabel());

		try {
			transitionSystem.setLabel("234!\"§12");
		} catch (IllegalArgumentException ex) {
			return;
		}

		fail("Missing exception");
	}
}
